(function () {
"use strict";
const APP_NAME = CB_APP_NAME;
const KNOWN_BROWSER_TOOLBAR_IDS = {
"toolbar-menubar": 1,
"nav-bar": 1,
"PersonalToolbar": 1,
"TabsToolbar": 1,
"addon-bar": 1,
"devtools-sidebar-toolbar": 1,
"inspector-toolbar": 1};
var {classes: Cc, interfaces: Ci} = Components;
const XULNS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
function OverlayController() {
this._barCore = Cc["@yandex.ru/custombarcore;" + this._appName].getService().wrappedJSObject;
this._barCore.Lib.sysutils.freezeObj(KNOWN_BROWSER_TOOLBAR_IDS);
this._application = this._barCore.application;
this._logger = this._application.getLogger("OverlayController");
this._urlBarItems = {
};
this._loadedStylesheets = Object.create(null);
this._windowDataIsland = Object.create(null);
window.addEventListener("DOMContentLoaded",this,false);
}

OverlayController.prototype = {
get window() window,
get appToolbar() this._appToolbar,
get navToolbar() document.getElementById("nav-bar"),
get chevronButton() document.getElementById("toggle-barless-button"),
get windowListener() this._windowListener,
get application() this._application,
get guiBuilder() this._guiBuilder,
get widgetHost() this._widgetHost,
get windowDataIsland() this._windowDataIsland,
applyPreset: function WndCtrl_applyPreset(preset, applyMode, forceSettings, ignoreWidgetErrors) {
const validModes = {
__proto__: null,
hard: 1,
soft: 1,
order: 1,
addleft: 1,
addright: 1};
const sysutils = this._barCore.Lib.sysutils;
const strutils = this._barCore.Lib.strutils;
if (! (preset instanceof this._application.BarPlatform.Preset))
throw new this._barCore.Lib.CustomErrors.EArgType("preset", "Preset", preset);
if (! (applyMode in validModes))
throw new this._barCore.Lib.CustomErrors.EArgRange("applyMode", "'hard', 'soft', 'order', 'addleft', 'addright'", applyMode);
var isAddMode = applyMode == "addleft" || applyMode == "addright";
this._logger.debug(strutils.formatString("Applying preset '%1' in '%2' mode. Force: %3",[preset.address, applyMode, forceSettings]));
const widgetLibrary = this._application.widgetLibrary;
const thisToolbar = this._appToolbar;
var sourceToolbars = {
};
sourceToolbars[thisToolbar.id] = thisToolbar;
var lastPlacedElement;
var pivotElement = isAddMode ? applyMode == "addleft" ? this.getToolbarSpring() : null : thisToolbar.firstChild;
const controller = this;
const separators = {
"http://bar.yandex.ru/packages/yandexbar#separator": "toolbarseparator",
"http://bar.yandex.ru/packages/yandexbar#spring": "toolbarspring",
"http://bar.yandex.ru/packages/yandexbar#spacer": "toolbarspacer"};
var reuseLists = {
};
function findReusableItems(widgetID) {
var sepElemName = separators[widgetID];
if (sepElemName)
{
return reuseLists[widgetID] || (reuseLists[widgetID] = Array.slice(controller._appToolbar.querySelectorAll(sepElemName)));
}

return reuseLists[widgetID] || (reuseLists[widgetID] = controller.getWidgetItems(widgetID));
}

for(let [, widgetEntry] in Iterator(preset.visibleWidgetEntries)) {
try {
let widgetID = widgetEntry.componentID;
let widget, widgetItem, sourceToolbar;
this._logger.debug("Processing widget entry " + widgetID);
if (isAddMode)
{
[widgetItem, widget, sourceToolbar] = this._placeAnyWidget(widgetID,thisToolbar,pivotElement,false);
}
 else
{
let widgetInfo = widgetLibrary.isKnownWidget(widgetID) && widgetLibrary.getWidgetInfo(widgetID);
let reusedItems;
if (widgetInfo && ! widgetInfo.isUnique || widgetID in separators)
reusedItems = findReusableItems(widgetID);
[widgetItem, widget, sourceToolbar] = this._placeAnyWidget(widgetID,thisToolbar,pivotElement,false,reusedItems);
lastPlacedElement = widgetItem;
}

let justCreated = ! ! widgetItem && ! sourceToolbar;
if (sourceToolbar)
sourceToolbars[sourceToolbar.id] = sourceToolbar;
if (widgetItem)
pivotElement = widgetItem.nextSibling;
if (applyMode == "hard" || justCreated)
{
let entrySettings = widgetEntry.settings;
if (widget && entrySettings)
widget.applySettings(entrySettings,true);
}
 else
if (forceSettings)
{
let forcedSettings = widgetEntry.forcedSettings;
if (sysutils.isEmptyObject(forcedSettings))
continue;
if (! widget)
{
let widgetItems = findReusableItems(widgetID);
widgetItem = widgetItems.shift();
if (! widgetItem)
{
this._logger.warn("Can't force settings. Widget not found: " + widgetID);
continue;
}

widget = this._widgetHost.getWidget(widgetItem.wdgtInstanceID);
}

widget.applySettings(forcedSettings,true);
}

}
catch (e) {
if (! ignoreWidgetErrors)
throw e;
this._logger.error("Could not put preset widget to the toolbar. " + strutils.formatError(e));
this._logger.debug(e.stack);
}

}

if (applyMode == "hard" || applyMode == "soft")
{
let next;
while (next = lastPlacedElement && lastPlacedElement.nextSibling) this.removeItem(next,true);
}

for each(let [, toolbar] in Iterator(sourceToolbars)) this._persistToolbarSet(toolbar);
}
,
placeWidgets: function WndCtrl_placeWidgets(widgetEnties, rightPart, dontFail) {
if (! Array.isArray(widgetEnties))
throw new this._barCore.Lib.CustomErrors.EArgType("widgetEnties", "Array", widgetEnties);
if (! widgetEnties.length)
return;
var relativeElement;
if (rightPart)
{
widgetEnties.reverse();
relativeElement = this._appToolbar.lastChild;
}
 else
{
relativeElement = this.getToolbarSpring();
}

var modifiedToolbars = {
};
for (let i = 0, len = widgetEnties.length;i < len;i++) {
let widgetEntry = widgetEnties[i];
if (widgetEntry.enabled != widgetEntry.ENABLED_YES)
continue;
try {
let [widget, , srcToolbar] = this.placeWidget(widgetEntry.componentID,relativeElement,rightPart,"sourceToolbar");
if (widget && widgetEntry.settings)
widget.applySettings(widgetEntry.settings,true);
if (srcToolbar)
modifiedToolbars[srcToolbar.id] = srcToolbar;
}
catch (e) {
if (! dontFail)
throw e;
this._logger.error("Could not place WidgetEntry '" + widgetEntry.componentID + "'. " + e);
}

}

modifiedToolbars[this._appToolbar.id] = this._appToolbar;
for each(let [, toolbar] in Iterator(modifiedToolbars)) this._persistToolbarSet(toolbar);
}
,
placeWidget: function WndCtrl_placeWidget(widgetID, relativeTo, placeAfter, autoPersist, failUniqueExists, reuseItems) {
const possibleAutoPersistValues = {
__proto__: null,
no: 0,
sourceToolbar: 0,
all: 0};
autoPersist = autoPersist || "all";
if (! (autoPersist in possibleAutoPersistValues))
throw new this._barCore.Lib.CustomErrors.EArgRange("autoPersist", "'no', 'sourceToolbar', 'all'", autoPersist);
var [pivotElement, destToolbar] = this._determineItemDestination(relativeTo,placeAfter);
var [toolbarItem, widget, sourceToolbar] = this._placeAnyWidget(widgetID,destToolbar,pivotElement,failUniqueExists,reuseItems);
if (sourceToolbar && autoPersist !== "no" && sourceToolbar !== destToolbar)
this._persistToolbarSet(sourceToolbar);
if (autoPersist == "all")
this._persistToolbarSet(destToolbar);
return [widget, toolbarItem, sourceToolbar];
}
,
removeItem: function WndCtrl_removeItem(DOMElementOrWidgetInstanceID, dontPersist) {
var widgetElement;
if (DOMElementOrWidgetInstanceID instanceof Ci.nsIDOMElement)
widgetElement = DOMElementOrWidgetInstanceID; else
if (typeof DOMElementOrWidgetInstanceID == "string")
widgetElement = this._widgetHost.getToolbarElement(DOMElementOrWidgetInstanceID); else
throw new TypeError("Widget DOM element or instance ID string required");
var toolbar = widgetElement.parentNode;
var palette = this._toolbox.palette;
var widgetLibrary = this._application.widgetLibrary;
if (widgetElement.wdgtIsPlatformWidget)
{
if (widgetLibrary.isKnownWidget(widgetElement.wdgtPrototypeID) && widgetElement.wdgtIsUnique)
{
let widgetInfo = widgetLibrary.getWidgetInfo(widgetElement.wdgtPrototypeID);
let paletteItem = this._application.overlayProvider.makePaletteItem(document,widgetInfo,0);
if (! palette.getElementsByAttribute("id",paletteItem.id).length)
palette.appendChild(paletteItem);
}

widgetElement.wdgtKill();
toolbar.removeChild(widgetElement);
}
 else
{
widgetElement = toolbar.removeChild(widgetElement);
if (["toolbarseparator", "toolbarspacer", "toolbarspring"].indexOf(widgetElement.localName) == - 1)
{
if (! palette.getElementsByAttribute("id",widgetElement.id).length)
palette.appendChild(widgetElement);
}

}

if (! dontPersist)
this._persistToolbarSet(toolbar);
return toolbar;
}
,
removeWidgetsOfPackage: function WndCtrl_removeWidgetsOfPackage(packageID) {
var toolbarItems = this.getWidgetItemsByPackage(packageID);
for (let i = toolbarItems.length;i--;) this.removeItem(toolbarItems[i]);
}
,
removeWidgetsOfProto: function WndCtrl_removeWidgetsOfProto(protoID, fromToolbarID) {
var toolbarItems = this.getWidgetItems(protoID,fromToolbarID);
for (let i = toolbarItems.length;i--;) this.removeItem(toolbarItems[i]);
}
,
removeItemById: function WndCtrl_removeItemById(aDOMElementId, aDontPersist) {
this.removeItem(document.getElementById(aDOMElementId),aDontPersist);
}
,
checkNeedSetBarlessMode: function WndCtrl_checkNeedSetBarlessMode() {
if (! this.application.barless || this._toolbox.hasAttribute("cb-barless"))
return;
this._toolbox.setAttribute("cb-barless","true");
this._logger.debug("Activating barless mode...");
try {
this._moveToNavBar();
this._moveLogoWidget();
this._removeObsoleteElements();
this._cleanToolbarSeparators(this._appToolbar);
this._persistToolbarSet(this.navToolbar);
if (! this._appToolbar.collapsed && ! this._appToolbar.hasChildNodes())
{
this._appToolbar.collapsed = true;
document.persist(this._appToolbar.id,"collapsed");
}

}
catch (e) {
this._logger.error("Could not activate barless mode. " + e);
this._logger.debug(e.stack);
}

}
,
_moveToNavBar: function WndCtrl__moveToNavBar() {
this._logger.config("Moving items to nav-bar");
var allItems = this.getAllWidgetItems(true);
this._logger.debug("My toolbar contents: " + allItems.map(function (item) item.getAttribute("id")));
var urlBarContainer = document.getElementById("urlbar-container");
var relativeElement = urlBarContainer;
if (urlBarContainer)
{
["reload-button", "stop-button"].forEach(function (mergedBtnID) {
var next = relativeElement.nextSibling;
if (next && next.getAttribute("id") == mergedBtnID)
relativeElement = next;
}
);
}

var findReusableItems = (function findReusableItems(reuseLists, widgetID) {
return reuseLists[widgetID] || (reuseLists[widgetID] = this.getWidgetItems(widgetID));
}
).bind(this,{
});
var tryPlaceWidget = (function tryPlaceWidget(widgetEntry) {
try {
let reusableItems = findReusableItems(widgetEntry.componentID);
let numFoundWidgets = reusableItems.length;
let widget;
[widget, relativeElement] = this.placeWidget(widgetEntry.componentID,relativeElement,true,"sourceToolbar",false,reusableItems);
let widgetWasReused = numFoundWidgets > reusableItems.length;
let settingsToApply = widgetWasReused ? widgetEntry.forcedSettings : widgetEntry.settings;
if (widget && settingsToApply)
widget.applySettings(settingsToApply,true);
return true;
}
catch (e) {
this._logger.warn("Could not place widget " + widgetEntry.componentID + ". " + e);
this._logger.debug(e.stack);
}

return false;
}
).bind(this);
this._logger.debug("Moving all important visible widgets first...");
for each(let widgetEntry in this.application.defaultPreset.visibleWidgetEntries) {
if (! widgetEntry.isImportant)
continue;
tryPlaceWidget(widgetEntry);
}

var installInfo = this.application.addonManager.info;
var migratedToBarlessMode = this.application.preferences.get("migratedToBarlessMode",false);
if (! installInfo.isFreshAddonInstall && ! migratedToBarlessMode)
{
this.application.preferences.set("migratedToBarlessMode",true);
let navbar = this.navToolbar;
let favWidgetsMoved = 0;
let autoLimits = this.application.autoinstaller.currentLimits;
this._logger.debug("Moving most used widgets...");
let activeWidgetIDs = this.getAllWidgetItems().map(function (wi) wi.getAttribute("cb-proto-id"));
let usageList = this.application.componentsUsage.mapToUsageInfo(activeWidgetIDs);
for each(let usageObj in usageList) {
this._logger.debug("Widget " + usageObj.id + " average usage: " + usageObj.avgUsage);
for each(let widgetItem in this.getWidgetItems(usageObj.id)) {
if (favWidgetsMoved > autoLimits.total)
break;
this._putElement(widgetItem,navbar,relativeElement.nextSibling);
relativeElement = widgetItem;
favWidgetsMoved++;
}

}

this._logger.debug("Moved " + favWidgetsMoved + " most used widgets.");
if (favWidgetsMoved == 0)
{
this._logger.debug("Placing 'auto' widgets...");
let autoWidgetsPlaced = 0;
let entryGen = this.application.autoinstaller.genHistoryRelevantEntries(this.application.internalDefaultPreset);
for each(let widgetEntry in entryGen) {
if (widgetEntry.componentType != widgetEntry.TYPE_WIDGET)
continue;
if (tryPlaceWidget(widgetEntry))
autoWidgetsPlaced++;
}

this._logger.debug("Placed " + autoWidgetsPlaced + " 'auto' widgets.");
}

}
 else
{
this._logger.debug(this._barCore.Lib.strutils.formatString("Not moving anything else. isFreshAddonInstall: %1, migratedToBarlessMode: %2",[installInfo.isFreshAddonInstall, migratedToBarlessMode]));
}

}
,
_moveLogoWidget: function WndCtrl__moveLogoWidget() {
var urlBarContainer = document.getElementById("urlbar-container");
if (this._application.preferences.get("logoBeforeAddressBar",true) && urlBarContainer)
{
let logoItem = this.getWidgetItems("http://bar.yandex.ru/packages/yandexbar#logo")[0];
if (logoItem)
{
this._putElement(logoItem,urlBarContainer.parentNode,urlBarContainer);
}

}

}
,
_removeObsoleteElements: function WndCtrl__removeObsoleteElements() {
var disabledBarlessElements = ["http://bar.yandex.ru/packages/yandexbar#search"];
disabledBarlessElements.forEach(function (protoId) this.removeWidgetsOfProto(protoId),this);
var navbar = this.navToolbar;
if (! navbar)
return;
var elementIdsToRemove = ["home-button"];
const smartboxID = "http://bar-widgets.yandex.ru/packages/approved/176/manifest.xml#smartbox";
var smartBoxEntry = this._application.defaultPreset.pluginIDs[smartboxID];
if (smartBoxEntry && smartBoxEntry.enabled == smartBoxEntry.ENABLED_YES)
elementIdsToRemove.push("search-container");
elementIdsToRemove.forEach(function (elementId) {
var existElement = document.getElementById(elementId);
if (existElement && existElement.parentNode === navbar)
this.removeItem(existElement);
}
,this);
}
,
clearToolbar: function WndCtrl_clearToolbar() {
Array.slice(this._appToolbar.childNodes).forEach(function (item) {
if (! this._isPermanetToolbarItem(item))
this.removeItem(item,true);
}
,this);
this._persistToolbarSet(this._appToolbar);
}
,
updatePalette: function WndCtrl_updatePalette(packageIDs) {
if (! Array.isArray(packageIDs))
throw new this._barCore.Lib.CustomErrors.EArgType("packageIDs", "Array", packageIDs);
var palette = this._toolbox.palette;
var items = palette.querySelectorAll("toolbaritem[cb-app='" + this._appName + "']");
for (let i = items.length;i--;) {
let item = items.item(i);
let protoID = item.getAttribute("cb-proto-id");
let [packageID] = this._application.BarPlatform.parseComponentID(protoID);
let itemInfo = this._application.overlayProvider.parseWidgetItemId(item.id);
let isNewPackage = packageIDs.indexOf(packageID) >= 0;
if (! isNewPackage)
continue;
if (itemInfo.isFromDefaultSet)
continue;
if (this._application.widgetLibrary.isKnownWidget(protoID))
{
let packageInfo = this._application.packageManager.getPackageInfo(packageID);
let banReason = this._application.packagesBlackList.banReasonForPackage(packageInfo);
if (! banReason)
continue;
}

item.parentNode.removeChild(item);
}

for (let idIndex = 0, idsLength = packageIDs.length;idIndex < idsLength;idIndex++) {
let packageID = packageIDs[idIndex];
let packageInfo = this._application.packageManager.getPackageInfo(packageID);
let banReason = this._application.packagesBlackList.banReasonForPackage(packageInfo);
if (banReason)
continue;
let widgetsInfo = this._application.widgetLibrary.getWidgetsInfo(packageID);
for (let i = 0, len = widgetsInfo.length;i < len;i++) {
let widgetInfo = widgetsInfo[i];
if (document.querySelector("[cb-proto-id='" + widgetInfo.id + "']"))
continue;
let widgetItem = this._application.overlayProvider.makePaletteItem(document,widgetInfo,0);
palette.appendChild(widgetItem);
}

}

}
,
appendToPalette: function WndCtrl_appendToPalette(newWidgetIDs) {
var palette = this._toolbox.palette;
for each(let widgetID in newWidgetIDs) {
let widgetInfo = this._application.widgetLibrary.getWidgetInfo(widgetID);
let widgetItem = this._application.overlayProvider.makePaletteItem(document,widgetInfo,0);
palette.appendChild(widgetItem);
}

}
,
deleteObsoleteWidgetItems: function WndCtrl_deleteObsoleteWidgetItems() {
var toolbars = Object.create(null);
var widgetLibrary = this._application.widgetLibrary;
var widgetItems = this.getWidgetItems();
for (let i = widgetItems.length;i--;) {
let widgetItem = widgetItems[i];
let widgetID = widgetItem.getAttribute("cb-proto-id");
if (widgetLibrary.isKnownWidget(widgetID) && ! this._getWidgetBanReason(widgetID))
continue;
let toolbar = widgetItem.parentNode;
toolbars[toolbar.id] = toolbar;
widgetItem.wdgtKill();
widgetItem.parentNode.removeChild(widgetItem);
}

for each(let toolbar in toolbars) this._persistToolbarSet(toolbar);
}
,
updateToolbarDefaultSet: function WndCtrl_updateToolbarDefaultSet() {
var palette = this._toolbox.palette;
var defSetItems = palette.querySelectorAll("toolbaritem[cb-app='" + this._appName + "'][cb-defaultset]");
for (let i = defSetItems.length;i--;) {
let defSetItem = defSetItems.item(i);
defSetItem.parentNode.removeChild(defSetItem);
}

var [newItems, newIDs] = this._application.overlayProvider.makeDefaultSetItems(document);
for (let i = 0, len = newItems.length;i < len;i++) palette.appendChild(newItems[i]);
this._appToolbar.setAttribute("defaultset",newIDs.join(","));
}
,
switchWidgets: function WndCtrl_switchWidgets(packageID, on) {
var widgetItems = this.getWidgetItemsByPackage(packageID);
for (let i = widgetItems.length;i--;) {
try {
widgetItems[i].wdgtSwitchOn(on);
}
catch (e) {
this._logger.error("Could not switch widget. " + this._barCore.Lib.strutils.formatError(e));
}

}

}
,
getToolbarSpring: function WndCtrl_getToolbarSpring() {
return this._appToolbar.querySelector("toolbarspring") || (this.application.barless ? null : this._appToolbar.insertItem("spring"));
}
,
getWidgetItems: function WndCtrl_getWidgetItems(protoID, fromToolbarID) {
var selector = (fromToolbarID ? "#" + fromToolbarID + " > " : "*:not(toolbarpalette) ") + "toolbaritem[cb-app='" + this._appName + "']" + (protoID ? "[cb-proto-id='" + protoID + "']" : "");
return Array.slice(document.querySelectorAll(selector));
}
,
getAllWidgetItems: function WndCtrl_getAllWidgetItems(withSeparators) {
var toolbarID = this._toolbarID;
var selectors = ["toolbaritem[cb-app='" + this._appName + "']"];
if (withSeparators)
{
selectors.push("toolbar[id='" + toolbarID + "'] toolbarseparator","toolbar[id='" + toolbarID + "'] toolbarspring","toolbar[id='" + toolbarID + "'] toolbarspacer");
}

var widgetItems = Array.slice(document.querySelectorAll(selectors.join(", ")));
return widgetItems;
}
,
getWidgetItemsByPackage: function WndCtrl_getWidgetItemsByPackage(packageID) {
var windowItems = this.getWidgetItems();
if (! packageID)
return windowItems;
return windowItems.filter(function filterByPkgID(item) {
return item.wdgtIsPlatformWidget && item.wdgtPrototypeID.indexOf(packageID) == 0;
}
);
}
,
get knownBrowserToolbarIDs() KNOWN_BROWSER_TOOLBAR_IDS,
getAlienToolbars: function WndCtrl_getAlienToolbars(includeOwnToolbar) {
var allToolbars = Array.slice(document.querySelectorAll("#navigator-toolbox toolbar"));
return allToolbars.filter((function alienToolbarFilter(toolbar) {
var tbID = toolbar.getAttribute("id");
var bTbHidden = toolbar.getAttribute("hidden") == "true";
return ! (tbID in KNOWN_BROWSER_TOOLBAR_IDS) && (includeOwnToolbar || tbID != this._toolbarID) && ! bTbHidden;
}
).bind(this));
}
,
widgetIsAlwaysVisible: function WndCtrl_widgetIsAlwaysVisible(widgetItem) {
var widgetParent = widgetItem.parentNode;
var parentToolbar = widgetParent && widgetParent.localName == "toolbar" && widgetParent;
return ! parentToolbar || parentToolbar.getAttribute("id") in KNOWN_BROWSER_TOOLBAR_IDS;
}
,
requireStylesheet: function WndCtrl_requireStylesheet(stylesheetURL) {
var ssUseInfo = this._loadedStylesheets[stylesheetURL];
if (! ssUseInfo)
{
let stylesheet = this._nativeStylesElement.sheet;
let i = stylesheet.insertRule("@import url('" + stylesheetURL + "');",stylesheet.cssRules.length);
this._loadedStylesheets[stylesheetURL] = ssUseInfo = {
cssRule: stylesheet.cssRules.item(i),
users: 0};
}

ssUseInfo.users++;
}
,
stylesheetNotNeeded: function WndCtrl_stylesheetNotNeeded(stylesheetURL) {
var ssUseInfo = this._loadedStylesheets[stylesheetURL];
if (! ssUseInfo)
return;
ssUseInfo.users--;
if (ssUseInfo.users < 1)
{
let cssRule = ssUseInfo.cssRule;
let parentStyleSheet = cssRule.parentStyleSheet;
let parentRules = parentStyleSheet.cssRules;
cssRule.styleSheet.disabled = true;
for (let i = 0, len = parentRules.length;i < len;i++) {
if (cssRule === parentRules.item(i))
{
parentStyleSheet.deleteRule(i);
break;
}

}

delete this._loadedStylesheets[stylesheetURL];
}

}
,
handleEvent: function WndCtrl_handleEvent(event) {
switch (event.type) {
case "DOMContentLoaded":
this._onContentReady(event);
break;
case "load":
this._onOverlayLoaded(event);
break;
case "unload":
this._onOverlayUnload(event);
break;
}

}
,
observe: function WndCtrl_observe(subject, topic, data) {
switch (topic) {
case this._barCore.eventTopics.EVT_PLUGIN_ENABLED:
this._tryApplyPluginResources(subject.wrappedJSObject);
break;
case this._barCore.eventTopics.EVT_PLUGIN_BEFORE_DISABLED:
this._tryRevertPluginResources(subject.wrappedJSObject);
break;
default:
this._logger.warn("Unexpected notification: " + [subject, topic, data]);
}

}
,
_appName: APP_NAME,
_barCore: null,
_application: null,
_logger: null,
_widgetHost: null,
_guiBuilder: null,
_urlBarItems: null,
_urlBarContainer: null,
_nativeStylesElement: null,
get _toolbarID() {
return this._application.name + "-bar";
}
,
_getWidgetBanReason: function WndCtrl__getWidgetBanReason(widgetID) {
var [packageID, widgetName] = this._application.BarPlatform.parseComponentID(widgetID);
var packageInfo = this._application.packageManager.getPackageInfo(packageID);
return this._application.packagesBlackList.banReasonForPackage(packageInfo);
}
,
_isPermanetToolbarItem: function WndCtrl__isPermanetToolbarItem(node) {
var importantComponentIDs = this._application.defaultPreset.importantComponentIDs;
if (node.getAttribute("cb-app") == this._application.name)
{
let importantEntry = importantComponentIDs[node.getAttribute("cb-proto-id")];
if (importantEntry && importantEntry.enabled == importantEntry.ENABLED_YES)
return true;
}

var nodeLocalName = node.localName;
if (["toolbarseparator", "toolbarspacer", "toolbarspring"].indexOf(nodeLocalName) != - 1)
{
let previousSiblingLocalName = node.previousSibling ? node.previousSibling.localName : null;
if (nodeLocalName != previousSiblingLocalName)
return true;
}

return false;
}
,
_onContentReady: function WndCtrl__onContentReady(domReadyEvent) {
domReadyEvent.currentTarget.removeEventListener("DOMContentLoaded",this,false);
var chromehiddenAttrVal = document.documentElement.getAttribute("chromehidden");
if (chromehiddenAttrVal.match(/\btoolbar\b/i))
return;
this._init();
}
,
_init: function WndCtrl__init() {
this._nativeStylesElement = document.createElementNS("http://www.w3.org/1999/xhtml","style");
this._nativeStylesElement.setAttribute("id",this._appName + "-native-stylesheets");
document.documentElement.appendChild(this._nativeStylesElement);
this._windowListener = new this._barCore.Lib.WindowListener(window, this._appName, this._application.getLogger("WindowListener"));
this._widgetHost = new WidgetHost(this);
this._guiBuilder = new this._application.XB.UI.Builder(this._widgetHost);
window.addEventListener("load",this,false);
}
,
_onOverlayLoaded: function WndCtrl__onOverlayLoaded(loadEvent) {
this._logger.config("Overlay loaded");
loadEvent.currentTarget.removeEventListener("load",this,false);
loadEvent.currentTarget.addEventListener("unload",this,false);
var urlBarIcons = document.getElementById("urlbar-icons");
if (urlBarIcons)
{
this._urlBarContainer = document.createElementNS(urlBarIcons.namespaceURI,"hbox");
this._urlBarContainer.setAttribute("id",this._appName + "-urlbar-icons");
urlBarIcons.appendChild(this._urlBarContainer);
}
 else
{
this._logger.warn("Can't find 'urlbar-icons' container.");
}

var activePlugins = this._application.widgetLibrary.getPlugins(undefined,true);
for (let i = activePlugins.length;i--;) {
this._logger.debug("Try apply plugin resources: " + activePlugins[i].id);
this._tryApplyPluginResources(activePlugins[i]);
}

this.checkNeedSetBarlessMode();
var obsService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
obsService.addObserver(this,this._barCore.eventTopics.EVT_PLUGIN_ENABLED,false);
obsService.addObserver(this,this._barCore.eventTopics.EVT_PLUGIN_BEFORE_DISABLED,false);
contextMenu.startup(this._application,this._logger);
this._application.onNewBrowserReady(this);
}
,
_onOverlayUnload: function WndCtrl__onOverlayUnload(unloadEvent) {
this._logger.info("Overlay unloads");
unloadEvent.currentTarget.removeEventListener("unload",this,false);
contextMenu.shutdown();
var obsService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
obsService.removeObserver(this,this._barCore.eventTopics.EVT_PLUGIN_ENABLED);
obsService.removeObserver(this,this._barCore.eventTopics.EVT_PLUGIN_BEFORE_DISABLED);
var activePlugins = this._application.widgetLibrary.getPlugins(undefined,true);
for (let i = activePlugins.length;i--;) {
this._logger.trace("Try revert plugin resources: " + activePlugins[i].id);
this._tryRevertPluginResources(activePlugins[i]);
}

this._widgetHost.clear();
this._guiBuilder.finalize();
window.document.documentElement.removeChild(this._nativeStylesElement);
this._loadedStylesheets = Object.create(null);
this._windowDataIsland = Object.create(null);
}
,
_persistToolbarSet: function WndCtrl__persistToolbarSet(toolbar) {
toolbar.setAttribute("currentset",toolbar.currentSet);
document.persist(toolbar.id,"currentset");
}
,
_tryApplyPluginResources: function WndCtrl__tryApplyPluginResources(plugin) {
try {
for each(let styleURL in plugin.browserStyles) {
this.requireStylesheet(styleURL);
}

for each(let itemInfo in plugin.urlBarItems) {
this._putURLBarItem(plugin,itemInfo.name,itemInfo.priority);
}

}
catch (e) {
let strutils = this._barCore.Lib.strutils;
this._logger.error(strutils.formatString("Could not apply plugin resources (%1). %2",[plugin.id, strutils.formatError(e)]));
this._logger.debug(e.stack);
}

}
,
_tryRevertPluginResources: function WndCtrl__tryRevertPluginResources(plugin) {
try {
for each(let styleURL in plugin.browserStyles) {
this.stylesheetNotNeeded(styleURL);
}

this._removePluginURLBarItems(plugin);
}
catch (e) {
let strutils = this._barCore.Lib.strutils;
this._logger.error(strutils.formatString("Could not revert plugin resources (%1). %2",[plugin.id, strutils.formatError(e)]));
}

}
,
_putURLBarItem: function WndCtrl__putURLBarItem(plugin, itemName, priority) {
if (! this._urlBarContainer)
{
this._logger.warn("Can't find 'urlbar-icons' container.");
return;
}

var itemID = plugin.id + "-" + itemName;
var item = document.getElementById(itemID);
if (item)
return;
item = document.createElementNS(XULNS,"hbox");
item.setAttribute("id",itemID);
item.setAttribute("cb-proto-id",plugin.id);
item.setAttribute("cb-app",this._appName);
var insertBefore = null;
var urlBarElements = this._urlBarContainer.childNodes;
for (let i = 0, len = urlBarElements.length;i < len;i++) {
let urlBarElement = urlBarElements.item(i);
let itemInfo = this._urlBarItems[urlBarElement.getAttribute("id")];
if (! itemInfo)
continue;
if (priority <= itemInfo.priority)
{
insertBefore = urlBarElement;
break;
}

}

this._urlBarContainer.insertBefore(item,insertBefore);
try {
let itemHandler = plugin.initURLBarItem(item,itemName);
this._urlBarItems[itemID] = {
name: itemName,
ownerID: plugin.id,
priority: priority,
domElement: item,
handler: itemHandler};
}
catch (e) {
this._logger.error("Could not init URL bar element handler. " + this._barCore.Lib.strutils.formatError(e));
this._logger.debug(e.stack);
}

}
,
_removePluginURLBarItems: function WndCtrl__removePluginURLBarItems(plugin) {
for(let itemID in this._urlBarItems) {
let itemInfo = this._urlBarItems[itemID];
if (! itemInfo || itemInfo.ownerID != plugin.id)
continue;
try {
itemInfo.handler.finalize();
}
catch (e) {
this._logger.error("URLBar item handler finalize method failed. " + this._barCore.Lib.strutils.formatError(e));
}
 finally {
this._urlBarItems[itemID] = undefined;
let item = itemInfo.domElement;
item.parentNode.removeChild(item);
}

}

}
,
get _toolbox() {
return document.getElementById("navigator-toolbox");
}
,
get _appToolbar() {
return document.getElementById(this._toolbarID);
}
,
_cleanToolbarSeparators: function WndCtrl__cleanToolbarSeparators(toolbar) {
["toolbarseparator + toolbarseparator, toolbarseparator:first-child", "toolbarseparator:last-child", "toolbarspring:last-child"].forEach(function (query) {
Array.slice(toolbar.querySelectorAll(query)).forEach(function (separator) this.removeItem(separator,true),this);
}
,this);
this._persistToolbarSet(toolbar);
}
,
_determineItemDestination: function WndCtrl__determineItemDestination(relativeTo, placeAfter) {
var pivotElement;
var destToolbar;
if (relativeTo)
{
let relativeToElement;
if (relativeTo instanceof Ci.nsIDOMNode)
{
let relParent = relativeTo.parentNode;
if (! relParent || typeof relParent.insertItem != "function")
throw new Error("Invalid relative element or its parent: " + relativeTo.nodeName);
relativeToElement = relativeTo;
}
 else
{
relativeToElement = this._widgetHost.getToolbarElement(relativeTo.id);
}

pivotElement = placeAfter ? relativeToElement.nextSibling : relativeToElement;
destToolbar = relativeToElement.parentNode;
}
 else
{
destToolbar = this._appToolbar;
}

return [pivotElement, destToolbar];
}
,
_placeAnyWidget: function WndCtrl__placeAnyWidget(widgetID, destToolbar, pivotElement, failUniqueExists, reuseItems) {
if (failUniqueExists && (! reuseItems || ! reuseItems.length) && this._application.widgetLibrary.getWidgetInfo(widgetID).isUnique && this.getWidgetItems(widgetID).length > 0)
throw new Error("Unique widget already exists: " + widgetID);
var [widget, toolbarItem, sourceToolbar] = this._placeLibraryWidget(widgetID,destToolbar,pivotElement,reuseItems);
return [toolbarItem, widget, sourceToolbar];
}
,
_placeLibraryWidget: function WndCtrl__placeLibraryWidget(protoID, destToolbar, insertBeforeElement, reuseItems) {
var toolbarItem;
var widget;
var sourceToolbar;
toolbarItem = reuseItems ? reuseItems.shift() : undefined;
if (toolbarItem)
{
let info = this._application.overlayProvider.parseWidgetItemId(toolbarItem.id,true);
widget = this._widgetHost.getWidget(info.instanceID);
}
 else
{
let widgetProto = this._application.widgetLibrary.getWidgetProto(protoID);
if (widgetProto.isUnique)
{
let spawnedIDs = widgetProto.spawnedIDs;
if (spawnedIDs.length && this._widgetHost.hasWidget(spawnedIDs[0]))
[widget, toolbarItem] = this._widgetHost.getWidgetAndElement(spawnedIDs[0]);
}

}

var alreadyHaveWidget = ! ! toolbarItem;
if (alreadyHaveWidget)
{
sourceToolbar = toolbarItem.parentNode;
this._putElement(toolbarItem,destToolbar,insertBeforeElement);
}
 else
{
let widgetInfo = this._application.widgetLibrary.getWidgetInfo(protoID);
toolbarItem = this._addNewFromPalette(widgetInfo,destToolbar,insertBeforeElement);
let info = this._application.overlayProvider.parseWidgetItemId(toolbarItem.id,true);
widget = this._widgetHost.getWidget(info.instanceID);
}

if (! widget || ! toolbarItem)
throw new Error("_placeLibraryWidget finishes with bad return values " + [widget, toolbarItem, sourceToolbar]);
return [widget, toolbarItem, sourceToolbar];
}
,
_addNewFromPalette: function WndCtrl__addNewFromPalette(widgetInfo, destToolbar, insertBeforeElement) {
var palette = this._toolbox.palette;
var paletteItemID = this._application.overlayProvider.compileWidgetItemId(widgetInfo.id,0);
if (! palette.getElementsByAttribute("id",paletteItemID).length)
{
let paletteItem = this._application.overlayProvider.makePaletteItem(document,widgetInfo,0);
palette.appendChild(paletteItem);
}

var hadChildren = destToolbar.hasChildNodes();
var insertedItem = destToolbar.insertItem(paletteItemID,insertBeforeElement);
if (destToolbar.id !== this._toolbarID)
{
return insertedItem;
}

if (! hadChildren)
{
destToolbar.collapsed = false;
document.persist(destToolbar.id,"collapsed");
}

return insertedItem;
}
,
_putElement: function WndCtrl_putElement(DOMElement, destToolbar, putBeforeElement) {
if (DOMElement !== putBeforeElement)
{
if (putBeforeElement)
destToolbar.insertBefore(DOMElement,putBeforeElement); else
destToolbar.appendChild(DOMElement);
}

}
};
function WidgetHost(overlayController) {
this._overlayController = overlayController;
this._app = this._overlayController.application;
this._hostID = "WHost" + this._app.overlayProvider.genWidgetHostID();
this._logger = this._app.getLogger(this._hostID);
this._widgetsMap = Object.create(null);
}

;
WidgetHost.prototype = {
constructor: WidgetHost,
get id() {
return this._hostID;
}
,
get overlayController() {
return this._overlayController;
}
,
get logger() {
return this._logger;
}
,
navigate: function WidgetHost_navigate(navigateData, statActionID, widgetInstance) {
navigateData.sourceWindow = this._overlayController.window;
if (statActionID)
{
let widgetProto = widgetInstance.prototype;
if (this._app.isPrivilegedPackageURL(widgetProto.unit.unitPackage.id))
this._app.componentsUsage.logShortAction(statActionID); else
this._app.componentsUsage.logCustomAction(widgetProto.id,statActionID);
}

this._app.BarPlatform.navigateBrowser(navigateData);
}
,
setupWidget: function WidgetHost_setupWidget(widgetInstanceOrToolbarItemOrId) {
var id = typeof widgetInstanceOrToolbarItemOrId == "string" ? widgetInstanceOrToolbarItemOrId : widgetInstanceOrToolbarItemOrId.id;
this._app.openSettingsDialog(this._overlayController.window,id);
}
,
addWidget: function WidgetHost_addWidget(widgetID, relativeTo, placeAfter) {
return this._overlayController.placeWidget(widgetID,relativeTo,placeAfter,"all",true);
}
,
removeWidget: function WidgetHost_removeWidget(WIID) {
this._overlayController.removeItem(WIID);
}
,
clear: function WidgetHost_clear() {
for(let WIID in this._widgetsMap) {
try {
this.destroyWidget(WIID);
}
catch (e) {
this._logger.error(this._app.core.Lib.strutils.formatError(e));
}

}

this._widgetsMap = Object.create(null);
this._overlayController = null;
}
,
hasWidget: function WidgetHost_hasWidget(widgetIID) {
return widgetIID in this._widgetsMap;
}
,
createWidget: function WidgetHost_createWidget(protoID, instID, settings, boundToolbarElement) {
if (! instID)
throw new this._app.core.Lib.CustomErrors.EArgRange("instID", "/.+/", instID);
var widget;
if (! this.hasWidget(instID))
{
let widgetPrototype = this._app.widgetLibrary.getWidgetProto(protoID);
widget = widgetPrototype.createInstance(instID,this,settings);
this._widgetsMap[instID] = {
widget: widget,
toolbarElement: boundToolbarElement};
widget.buildUI(boundToolbarElement);
}
 else
{
this._logger.warn("Widget with this ID (" + instID + ") already exists. Will use the old one.");
widget = this.getWidget(instID);
widget.applySettings(settings,true);
}

return widget;
}
,
destroyWidget: function WidgetHost_destroyWidget(instID, eraseSettingsIfNeeded) {
var widget = this.getWidget(instID);
try {
if (eraseSettingsIfNeeded)
{
let widgetProto = widget.prototype;
if (widgetProto.isUnique && widgetProto.spawnedIDs.length == 1)
{
try {
widget.eraseSettings();
}
catch (e) {
this._logger.error("Could not erase widget setting. " + this._app.core.Lib.strutils.formatError(e));
}

}

}

widget.finalize();
}
 finally {
delete this._widgetsMap[instID];
}

}
,
getWidget: function WidgetHost_getWidget(widgetIID) {
return this._getWidgetPair(widgetIID).widget;
}
,
getToolbarElement: function WidgetHost_getToolbarElement(widgetIID) {
return this._getWidgetPair(widgetIID).toolbarElement;
}
,
getWidgetAndElement: function WidgetHost_getWidgetAndElement(widgetIID) {
var widgetPair = this._getWidgetPair(widgetIID);
return [widgetPair.widget, widgetPair.toolbarElement];
}
,
_consts: {
ERR_WIDGET_NOT_FOUND: "Couldn't find widget with this ID (%1)",
ERR_CANT_CONSTRUCT_WIDGET: "Failed creating widget"},
_app: null,
_logger: null,
_hostID: undefined,
_overlayController: null,
_widgetsMap: null,
_getWidgetPair: function WidgetHost__getWidgetPair(widgetIID) {
var widgetPair = this._widgetsMap[widgetIID];
if (! widgetPair)
{
let errmsg = this._app.core.Lib.strutils.formatString(this._consts.ERR_WIDGET_NOT_FOUND,[widgetIID]);
try {
throw new Error(errmsg);
}
catch (e) {
this._logger.error(errmsg);
this._logger.debug(e.stack);
throw e;
}

}

return widgetPair;
}
};
const contextMenu = {
startup: function YaContextMenu_startup(application, logger) {
this._barApp = application;
this._logger = logger;
let (menu = this._pageContextMenu) {
if (menu)
{
menu.addEventListener("command",this,false);
menu.addEventListener("popupshowing",this,false);
menu.addEventListener("popuphidden",this,false);
}

}

this._barApp.branding.addListener("package updated",this);
}
,
shutdown: function YaContextMenu_shutdown() {
this._barApp.branding.removeListener("package updated",this);
let (menu = this._pageContextMenu) {
if (menu)
{
menu.removeEventListener("command",this,false);
menu.removeEventListener("popupshowing",this,false);
menu.removeEventListener("popuphidden",this,false);
}

}

}
,
MENUITEMS_ID_PREFIX: APP_NAME + "-contextMenuIdPrefix-",
get _pageContextMenu() {
return document.getElementById("contentAreaContextMenu");
}
,
get _hiddenItem() {
var hiddenItemId = this.MENUITEMS_ID_PREFIX + "hidden";
var hiddenItem = document.getElementById(hiddenItemId);
if (! hiddenItem)
{
hiddenItem = document.createElement("menuseparator");
hiddenItem.setAttribute("id",hiddenItemId);
hiddenItem.setAttribute("hidden","true");
["context-keywordfield", "context-searchselect", "frame-sep"].some(function (id) {
var item = this.querySelector("#" + id);
if (! item)
return false;
this.insertBefore(hiddenItem,item);
return true;
}
,this._pageContextMenu) || this._pageContextMenu.appendChild(hiddenItem);
}

return hiddenItem;
}
,
_lastWord: null,
_getBrowserSelection: function YaContextMenu__getBrowserSelection() {
var focusedWindow = document.commandDispatcher.focusedWindow;
var selection = focusedWindow ? focusedWindow.getSelection().toString() : "";
return this._cropSelectionText(selection);
}
,
MAX_QUERY_LENGTH: 400,
_cropSelectionText: function YaContextMenu__cropSelectionText(aString) {
if (aString)
{
if (aString.length > this.MAX_QUERY_LENGTH)
{
let pattern = new RegExp("^(?:\\s*.){0," + this.MAX_QUERY_LENGTH + "}");
pattern.test(aString);
aString = RegExp.lastMatch;
}

aString = aString.trim().replace(/\s+/g," ");
if (aString.length > this.MAX_QUERY_LENGTH)
aString = aString.substr(0,this.MAX_QUERY_LENGTH);
}

return aString;
}
,
handleEvent: function YaContextMenu_handleEvent(aEvent) {
switch (aEvent.type) {
case "command":
this._onCommandEvent(aEvent);
break;
case "popupshowing":
if (aEvent.target == this._pageContextMenu)
this._onContextMenuPopupShowing();
break;
case "popuphidden":
if (aEvent.target == this._pageContextMenu)
this._onContextMenuPopupHidden();
break;
}

}
,
_onContextMenuPopupShowing: function YaContextMenu__onContextMenuPopupShowing() {
var selectedText = "";
var cmTarget = gContextMenu.target;
if (gContextMenu.isContentSelected)
{
selectedText = this._getBrowserSelection();
}
 else
if (gContextMenu.onTextInput && ! cmTarget.readOnly && cmTarget.type != "password")
{
if (cmTarget.value)
{
selectedText = cmTarget.value.toString().slice(cmTarget.selectionStart,cmTarget.selectionEnd);
selectedText = this._cropSelectionText(selectedText);
}

}

this._lastWord = selectedText;
if (selectedText)
{
if (selectedText.length > 15)
selectedText = selectedText.substr(0,15) + "...";
let templateParams = {
packagePath: this._barApp.name + "://branding/",
searchTermsCropped: selectedText,
contextType: "selectedText"};
let docFrag = this._transformToXULMenu(this._browserConfXML,templateParams);
this._pageContextMenu.insertBefore(docFrag,this._hiddenItem);
}

document.getElementById("context-searchselect").hidden = true;
}
,
_onContextMenuPopupHidden: function YaContextMenu__onContextMenuPopupHidden() {
this._lastWord = null;
var elements = this._pageContextMenu.getElementsByAttributeNS(this.BAR_NS,"contextType","selectedText");
var i = elements.length;
while (i--) {
let el = elements.item(i);
el.parentNode.removeChild(el);
}

}
,
BAR_NS: "http://bar.yandex.ru/firefox/",
DEFAULT_QUERY_CHARSET: "ISO-8859-1",
_textToSubURI: Cc["@mozilla.org/intl/texttosuburi;1"].getService(Ci.nsITextToSubURI),
_convertQueryForURI: function YaContextMenu__convertQueryForURI(aData, aQueryCharset) {
try {
return this._textToSubURI.ConvertAndEscape(aQueryCharset || this.DEFAULT_QUERY_CHARSET,aData);
}
catch (ex) {

}

return this._textToSubURI.ConvertAndEscape(this.DEFAULT_QUERY_CHARSET,aData);
}
,
_onCommandEvent: function YaContextMenu__onCommandEvent(aEvent) {
var target = aEvent.target;
if (! this._lastWord || ! target || target.getAttributeNS(this.BAR_NS,"contextType") != "selectedText")
return;
var url = target.getAttributeNS(this.BAR_NS,"linkURL");
if (! url)
return;
var encoding = target.getAttributeNS(this.BAR_NS,"linkEncoding");
var templateValues = {
searchTerms: this._convertQueryForURI(this._lastWord,encoding)};
this._barApp.core.Lib.misc.navigateBrowser({
url: this._barApp.branding.expandBrandTemplates(url,templateValues),
eventInfo: aEvent,
target: "new tab"});
var action = target.getAttributeNS(this.BAR_NS,"linkAction");
if (action)
this._barApp.componentsUsage.logShortAction(action);
}
,
_transformToXULMenu: function YaContextMenu__transformToXULMenu(xmlDoc, aParams) {
var Lib = this._barApp.core.Lib;
var xslStream = this._barApp.addonFS.getStream("$content/overlay/context-menu.xsl");
var xsl = Lib.fileutils.xmlDocFromStream(xslStream,null,null,true);
return Lib.xmlutils.transformXMLToFragment(xmlDoc,xsl,document,aParams);
}
,
__browserConfXML: null,
get _browserConfXML() {
if (! this.__browserConfXML)
this.__browserConfXML = this._barApp.branding.brandPackage.getXMLDocument("/browser/browserconf.xml");
return this.__browserConfXML;
}
,
observe: function YaContextMenu_observe(subject, topic, data) {
if (subject != this._barApp.branding || topic != "package updated")
return;
this._logger.info("Invalidating branded context menu data...");
this.__browserConfXML = null;
}
};
window[APP_NAME + "OverlayController"] = new OverlayController();
}
)();
